! =================================================================================================================================
! MODULE       : control
!
! CONTACT      : orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF        "control" module contains subroutines to initialize run time control parameters. 
!!
!!\n DESCRIPTION: 
!!
!! SVN          :
!! $HeadURL: 
!! $Date:  
!! $Revision: 
!! \n
!_ ================================================================================================================================

MODULE control
  
  USE constantes_soil
  USE constantes_var
  USE pft_parameters
  USE vertical_soil

  IMPLICIT NONE

CONTAINS
!! ================================================================================================================================
!! SUBROUTINE   : control_initialize 
!!
!>\BRIEF        This subroutine reads the configuration flags which control the behaviour of the model
!!              This subroutine was previsouly named intsurf_config and located in intersurf module. 
!!
!! DESCRIPTION  : None
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S): None
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE control_initialize(dt)

    IMPLICIT NONE
    
    REAL(r_std), INTENT(in)                    :: dt                    !! Time step in seconds
    INTEGER(i_std)                             :: jv                    !! Local index variable
    INTEGER(i_std)                             :: ier                   !! Error handeling
    LOGICAL                                    :: hydrol_cwrr_test      !! Temporary test variable
    LOGICAL                                    :: ok_co2_test           !! Temporary test variable
    LOGICAL                                    :: ok_explicitsnow_test  !! Temporary test variable
    LOGICAL                                    :: conflict              !! Check whether there are conflicts
    LOGICAL                                    :: temp                  !! Temp to check for conflicts  

    ! Archive the sechiba time step into module constantes_var 
    dt_sechiba=dt

    ! control initialisation with sechiba
    ok_sechiba = .TRUE.

    ! Start reading options from parameter file 

    !Config Key   = SOILTYPE_CLASSIF
    !Config Desc  = Type of classification used for the map of soil types 
    !Config Def   = zobler
    !Config If    = !IMPOSE_VEG
    !Config Help  = The classification used in the file that we use here 
    !Config         There are three classification supported:  
    !Config         Zobler (7 converted to 3) and USDA (12) 
    !Config Units = [-]
    !
    !-tdo- Suivant le type de classification utilisee pour le sol, on adapte nscm 
    soil_classif = 'zobler'
    CALL getin_p('SOILTYPE_CLASSIF',soil_classif)
    SELECTCASE (soil_classif)
    CASE ('zobler','none')
       nscm = nscm_fao
    CASE ('usda')
       nscm = nscm_usda
    CASE DEFAULT
       WRITE(numout,*) "Unsupported soil type classification: soil_classif=",soil_classif
       WRITE(numout,*) "Choose between zobler, usda and none according to the map"
       CALL ipslerr(3,'control_initialize','Bad choice of soil_classif',&
            'Choose between zobler, usda and none','')
    ENDSELECT
    WRITE(numout,*)'soil_classif,nscm', soil_classif, nscm

    !Config Key   = OK_BARE_SOIL_NEW
    !Config Desc  = Flag that controls the view on and calculation of bare soil
    !Config If    = OK_SECHIBA or OK_STOMATE 
    !Config Def   = TRUE
    !Config Help  = Choose between the two options to calculate the bare soil.
    !Config         False = classic view: gaps within a canopy should be treated
    !Config         bare soil. True = ecological view: gaps within a canopy are
    !Config         part of the ecosystem and should be treated as such.
    !Config Units = [-]
    CALL getin_p('OK_BARE_SOIL_NEW', ok_bare_soil_new) 

    !Config Key   = MULTI_LAYER_CONTROL
    !Config Desc  = A flag that controls severeal other flags related to the multi layer settings
    !Config Def   = y
    !Config Help  = Flag that automatically controls several other flags related
    !Config Help    to multi-layering (1/2/3). 1 - single layer: ok_hydrol_arch,
    !Config Help    ok_gs_feedback, ok_impose_canopy_structure  and and ok_mleb all
    !Config Help    TRUE, but The energy budget is only calculated for a single
    !Config Help    layer (jnlvls=1,jnlvls_under=0,jnlvls_canopy=1,jnlvls_over=0).
    !Config Help    No mleb output, ok_mleb_history_file is set to  FALSE.
    !Config Help    2 - multi-layer: ok_hydrol_arch, ok_gs_feedback
    !Config Help    and ok_mleb all TRUE. ok_impose_canopy_structure is False, and the
    !Config Help    energy budget is calculated for multiple layers 
    !Config Help    (jnlvls=29,jnlvls_under=10,jnlvls_canopy=10,jnlvls_over=9).    
    !Config Help    No mleb output, ok_mleb_history_file is set to  FALSE.
    !Config Help    3 user specific: user specific settings for these
    !Config Help    controls and layers as defined in the run.def by the user.
    !Config Units = [FLAG]
    MULTI_LAYER_CONTROL = 1 
    CALL getin_p('MULTI_LAYER_CONTROL', multi_layer_control)

    ! set flags according to the multi_layer_control flag.

    IF( multi_layer_control .EQ. 1 ) THEN  ! single layer

       ok_hydrol_arch = .TRUE.
       ok_gs_feedback = .TRUE.
       ok_mleb = .TRUE.
       ok_impose_can_structure = .TRUE.
       ok_mleb_history_file = .FALSE.

       ! Search for possible conflicts
       conflict = .FALSE.
       CALL getin_p('OK_HYDROL_ARCH', temp)
       IF (ok_hydrol_arch .NE. temp) conflict = .TRUE.
       CALL getin_p('OK_GS_FEEDBACK', temp)
       IF (ok_gs_feedback .NE. temp) conflict = .TRUE.
       CALL getin_p('OK_MLEB', temp)
       IF (ok_mleb .NE. temp) conflict = .TRUE.
       CALL getin_p('OK_IMPOSE_CAN_STRUCTURE', temp)
       IF (ok_impose_can_structure .NE. temp) conflict = .TRUE.
       CALL getin_p('OK_MLEB_HISTORY_FILE', temp)
       IF (ok_mleb_history_file .NE. temp) conflict = .TRUE.
       IF (conflict) THEN
          CALL ipslerr(2,'control_initialize', &
               'Some of the parameter values set in the', &
               'run.def or set as a default are overwritten by the',&
               'values implied by the MULTI_LAYER_CONTROL flag')
       ENDIF

    ELSEIF( multi_layer_control .EQ. 2 ) THEN  ! multi-layer

       ok_hydrol_arch = .TRUE.
       ok_gs_feedback = .TRUE.
       ok_mleb = .TRUE.
       ok_impose_can_structure = .FALSE.
       ok_mleb_history_file = .FALSE.

       ! Search for conflicts
       conflict = .FALSE.
       CALL getin_p('OK_HYDROL_ARCH', temp)
       IF (ok_hydrol_arch .NE. temp) conflict = .TRUE.
       CALL getin_p('OK_GS_FEEDBACK', temp)
       IF (ok_gs_feedback .NE. temp) conflict = .TRUE.
       CALL getin_p('OK_MLEB', temp)
       IF (ok_mleb .NE. temp) conflict = .TRUE.
       CALL getin_p('OK_IMPOSE_CAN_STRUCTURE', temp)
       IF (ok_impose_can_structure .NE. temp) conflict = .TRUE.
       CALL getin_p('OK_MLEB_HISTORY_FILE', temp)
       IF (ok_mleb_history_file .NE. temp) conflict = .TRUE.
       IF (conflict) THEN
          CALL ipslerr(2,'control_initialize', &
               'Some of the parameter values set in the', &
               'run.def or set as a default are overwritten by the',&
               'values implied by the MULTI_LAYER_CONTROL flag')
       ENDIF

    ELSEIF( multi_layer_control .EQ. 3 ) THEN 

       !Config Key   = OK_HYDROL_ARCH
       !Config Desc  = Activates the hydraulic architecture
       !Config If    = OK_SECHIBA
       !Config Def   = y
       !Config Help  = Flag that activates the hydraulic architecture routine (true/false)
       !Config Help    The trunk version of ORCHIDEE (false) uses soil water as a 
       !Config Help    proxy for water stress and applies the stress to Vcmax.
       !Config Help    When set to true the hydraulic architecture of the vegetation
       !Config Help    is accounted for to calculate the amount of water that 
       !Config Help    can be transported through the plant given the soil and leaf
       !Config Help    potential and the conductivities of the roots, wood and 
       !Config Help    leaves. Water supply through the plant is compared against
       !Config Help    the atmospheric demand for water. If the supply is smaller
       !Config Help    then the demand, the plant experiences water stress and the
       !Config Help    stomata will be closed (water stress is now on gs rather 
       !Config Help    than Vcmax). Note that whether stomatal regulation is used or 
       !Config Help    not is controled by a separate flag: ok_gs_feedback.
       !Config Units = [FLAG]
       ok_hydrol_arch = .TRUE.
       CALL getin_p('OK_HYDROL_ARCH', ok_hydrol_arch)

       !Config Key   = OK_GS_FEEDBACK
       !Config Desc  = Debug option for OK_HYDROL_ARCH  
       !Config If    = OK_SECHIBA, OK_HYDROL_ARCH
       !Config Def   = y
       !Config Help  = Flag that activates water stress on stomata (true/false)
       !Config Help    This flag is for debugging only! It allows developers
       !Config Help    to calculate GPP without any water stress. If the model is 
       !Config Help    used in production mode and ok_hydrol_arch is true this 
       !Config Help    flag should be true as well.
       !Config Units = [FLAG]
       ok_gs_feedback = .TRUE.
       CALL getin_p('OK_GS_FEEDBACK', ok_gs_feedback)

       !Config Key   = OK_MLEB
       !Config Desc  = Activate multi-layer energy budget  
       !Config If    = OK_SECHIBA
       !Config Def   = y
       !Config Help  = Flag that activates the multilayer energy budget (true/false)
       !Config Help    The model uses 10 (default) canopy layers to calculate
       !Config Help    the albedo, transmittance, absorbance and GPP. These canopy
       !Config Help    layers can be combined with 10 (default) layers below and 
       !Config Help    10 layers above the canopy to calculate the energy budget
       !Config Help    (ok_mleb=y). If set to no, this flag will make the model
       !Config Help    use 10 layers for the canopy albedo, transmittance, 
       !Config Help    absorbance and GPP and just a single layer for the energy 
       !Config Help    budget.
       !Config Units = [FLAG]
       ok_mleb = .FALSE.
       CALL getin_p('OK_MLEB', ok_mleb)

       !+++CHECK+++
       ! This may no longer be needed. One could get the same functionality
       ! by reading an biomass map in shechiba (lai_map) and not using stomate.
       !Config Key   = OK_IMPOSE_CAN_STRUCTURE
       !Config Desc  = Debug option for OK_MLEB 
       !Config If    = OK_SECHIBA, OK_MLEB
       !Config Def   = n
       !Config Help  = This flag is for debugging only! It allows developers
       !Config Help    to use a prescribed canopy structure rather then the 
       !Config Help    structure calculate by ORCHIDEE. The flag activates the 
       !Config Help    sections of code which directly link the energy budget 
       !Config Help    scheme to the the size and LAI profile of the canopy for the 
       !Config Help    respective PFT and age class that is calculated in stomate, 
       !Config Help    for the albedo. If set to TRUE and the multi-layer budget 
       !Config Help    is activated the model takes LAI profile information and 
       !Config Help    canopy level heights from the run.def. If set to FALSE, and 
       !Config Help    and the multi-layer energy budget is used the profile 
       !Config Help    infomation and canopy levels heights comes from the
       !Config Help    PGap-based processes for calculation of stand profile 
       !Config Help    information in stomate.
       !Config Units = [FLAG]
       ok_impose_can_structure = .FALSE.
       CALL getin_p('OK_IMPOSE_CAN_STRUCTURE', ok_impose_can_structure)
       !++++++++++

       !Config Key   = MLEB_NETCDF_FLAG
       !Config Desc  = Debug option for OK_MLEB 
       !Config If    = OK_SECHIBA, OK_MLEB
       !Config Def   = n
       !Config Help  = Flag that controls the writing of an output file with the 
       !Config Help    multi-layer energy simulations (true/false). Note that this
       !Config Help    a large file and writing it slows down the code.+++CHECK constantes_var+++
       !Config Units = [FLAG]
       ok_mleb_history_file = .FALSE.
       CALL getin_p('OK_MLEB_HISTORY_FILE', ok_mleb_history_file)

    ENDIF ! (multi_layer_control)

    WRITE(numout,*) 'multi_layer_control', multi_layer_control
    WRITE(numout,*) 'ok_hydrol_arch', ok_hydrol_arch
    WRITE(numout,*) 'ok_gs_feedback', ok_gs_feedback
    WRITE(numout,*) 'ok_mleb', ok_mleb
    WRITE(numout,*) 'ok_impose_can_structure', ok_impose_can_structure
    WRITE(numout,*) 'ok_mleb_history_file',ok_mleb_history_file       

    !Config Key   = RIVER_ROUTING
    !Config Desc  = Decides if we route the water or not
    !Config If    = OK_SECHIBA
    !Config Def   = n
    !Config Help  = This flag allows the user to decide if the runoff
    !Config         and drainage should be routed to the ocean
    !Config         and to downstream grid boxes.
    !Config Units = [FLAG]
    river_routing = .FALSE.
    CALL getin_p('RIVER_ROUTING', river_routing)
    WRITE(numout,*) "RIVER routing is activated : ",river_routing

    ! Control for the option HYDROL_CWRR which is not longer existing in the model. 
    ! Check here if in run.def HYDROL_CWRR=n. If that's the case then stop the model and ask the user to remove the flag from run.def
    hydrol_cwrr_test = .TRUE.
    CALL getin_p('HYDROL_CWRR', hydrol_cwrr_test)
    IF (.NOT. hydrol_cwrr_test) THEN
       CALL ipslerr(3,'control_initialize',&
            'HYDROL_CWRR=n is set in run.def but this option does not exist any more in ORCHIDEE', &
            'Choisnel hydrolology has been removed and CWRR is now the only hydrology module in ORCHIDEE',&
            'Remove parameter HYDROL_CWRR from run.def')
    END IF
    
    !Config Key   = DO_IRRIGATION
    !Config Desc  = Should we compute an irrigation flux 
    !Config If    = RIVER_ROUTING 
    !Config Def   = n
    !Config Help  = This parameters allows the user to ask the model
    !Config         to compute an irigation flux. This performed for the
    !Config         on very simple hypothesis. The idea is to have a good
    !Config         map of irrigated areas and a simple function which estimates
    !Config         the need to irrigate.
    !Config Units = [FLAG]
    do_irrigation = .FALSE.
    IF ( river_routing ) CALL getin_p('DO_IRRIGATION', do_irrigation)
    
    !Config Key   = DO_FLOODPLAINS
    !Config Desc  = Should we include floodplains 
    !Config If    = RIVER_ROUTING 
    !Config Def   = n
    !Config Help  = This parameters allows the user to ask the model
    !Config         to take into account the flood plains and return 
    !Config         the water into the soil moisture. It then can go 
    !Config         back to the atmopshere. This tried to simulate 
    !Config         internal deltas of rivers.
    !Config Units = [FLAG]  
    do_floodplains = .FALSE.
    IF ( river_routing ) CALL getin_p('DO_FLOODPLAINS', do_floodplains)
    
    ! Control of option OK_EXPLICITSNOW which is not longer existing in the model. 
    ! Check here if in run.def OK_EXPLICITSNOW=n. If that's the case then stop the model and ask the user to remove the flag from run.def.
    ok_explicitsnow_test = .TRUE.
    CALL getin_p('OK_EXPLICITSNOW', ok_explicitsnow_test)
    IF (.NOT. ok_explicitsnow_test) THEN
       CALL ipslerr(3,'control_initialize',&
            'OK_EXPLICITSNOW=n is set in run.def but this option does not exist any more in ORCHIDEE', &
            'Explicit snow scheme is now always used in ORCHIDEE.',&
            'Remove parameter OK_EXPLICITSNOW from run.def')
    END IF

    !Config Key   = STOMATE_OK_STOMATE
    !Config Desc  = Activate STOMATE?
    !Config If    = OK_SECHIBA
    !Config Def   = n
    !Config Help  = set to TRUE if STOMATE is to be activated
    !Config Units = [FLAG]
    ok_stomate = .TRUE.
    CALL getin_p('STOMATE_OK_STOMATE',ok_stomate)
    IF (printlev>=1) WRITE(numout,*) 'STOMATE is activated: ',ok_stomate

    ! Control for the option STOMATE_OK_CO2 which is not longer existing in the model.
    ! Check here if in run.def STOMATE_OK_CO2=n. If that's the case then stop the model and ask the user to remove the flag from run.def
    ok_co2_test = .TRUE.
    CALL getin_p('STOMATE_OK_CO2', ok_co2_test) 
    IF (.NOT. ok_co2_test) THEN 
       CALL ipslerr(3,'control_initialize',& 
            'STOMATE_OK_CO2=n is set in run.def but this option does not exist any more in ORCHIDEE', & 
            'Calculation of beta coefficient using Jarvis formulation has been removed and Farquar formulation is now always used',& 
            'Remove parameter STOMATE_OK_CO2 from run.def') 
    END IF

    !
    !Config Key   = STOMATE_OK_NCYCLE 
    !Config Desc  = Activate dynamic N cycle 
    !Config If    = OK_STOMATE 
    !Config Def   = n 
    !Config Help  = set to TRUE if N cycle is to be activated 
    !Config Units = [FLAG]  
    ok_ncycle = .FALSE. 
    CALL getin_p('STOMATE_OK_NCYCLE',ok_ncycle) 
    WRITE(numout,*) 'N cycle is activated: ',ok_ncycle  
   
    !Config Key   = STOMATE_IMPOSE_CN
    !Config Desc  = Impose the CN ratio of leaves 
    !Config If    = OK_STOMATE 
    !Config Def   = n 
    !Config Help  = set to TRUE if IMPOSE_CN is to be activated 
    !Config Units = [FLAG] 
    impose_cn = .FALSE. 
    CALL getin_p('STOMATE_IMPOSE_CN',impose_cn) 
    WRITE(numout,*) 'CN ratio is imposed: ',impose_cn  
    
    !Config Key   = STOMATE_OK_DGVM
    !Config Desc  = Activate DGVM?
    !Config If    = OK_STOMATE
    !Config Def   = n
    !Config Help  = set to TRUE if DGVM is to be activated
    !Config Units = [FLAG]
    ok_dgvm = .FALSE.
    CALL getin_p('STOMATE_OK_DGVM',ok_dgvm)

    !Config Key   = CHEMISTRY_BVOC
    !Config Desc  = Activate calculations for BVOC
    !Config If    = OK_SECHIBA
    !Config Def   = n
    !Config Help  = set to TRUE if biogenic emissions calculation is to be activated
    !Config Units = [FLAG]
    ok_bvoc = .FALSE.
    CALL getin_p('CHEMISTRY_BVOC', ok_bvoc)
    WRITE(numout,*) 'Biogenic emissions: ', ok_bvoc

    IF ( ok_bvoc ) THEN 
       ok_leafage         = .TRUE. 
       ok_radcanopy       = .TRUE. 
       ok_multilayer      = .TRUE.
       ok_pulse_NOx       = .TRUE.
       ok_bbgfertil_NOx   = .TRUE.
       ok_cropsfertil_NOx = .TRUE.

       ! Search for conflicts
       conflict = .FALSE.
       CALL getin_p('CHEMISTRY_LEAFAGE', temp)
       IF (ok_leafage .NE. temp) conflict = .TRUE.
       CALL getin_p('CANOPY_EXTINCTION', temp)
       IF (ok_radcanopy .NE. temp) conflict = .TRUE.
       CALL getin_p('CANOPY_MULTILAYER', temp)
       IF (ok_multilayer .NE. temp) conflict = .TRUE.
       CALL getin_p('NOx_RAIN_PULSE', temp)
       IF (ok_pulse_NOx .NE. temp) conflict = .TRUE.
       CALL getin_p('NOx_BBG_FERTIL', temp)
       IF (ok_bbgfertil_NOx .NE. temp) conflict = .TRUE.
       CALL getin_p('NOx_FERTILIZERS_USE', temp)
       IF (ok_cropsfertil_NOx .NE. temp) conflict = .TRUE.
       IF (conflict) THEN
          CALL ipslerr(2,'control_initialize', &
               'Some of the parameter values set in the', &
               'run.def or set as a default are overwritten by the',&
               'values implied by the OK_BVOC flag')
       ENDIF

    ELSE

       ok_leafage         = .FALSE. 
       ok_radcanopy       = .FALSE. 
       ok_multilayer      = .FALSE.
       ok_pulse_NOx       = .FALSE.
       ok_bbgfertil_NOx   = .FALSE.
       ok_cropsfertil_NOx = .FALSE.

       ! Search for conflicts
       conflict = .FALSE.
       CALL getin_p('CHEMISTRY_LEAFAGE', temp)
       IF (ok_leafage .NE. temp) conflict = .TRUE.
       CALL getin_p('CANOPY_EXTINCTION', temp)
       IF (ok_radcanopy .NE. temp) conflict = .TRUE.
       CALL getin_p('CANOPY_MULTILAYER', temp)
       IF (ok_multilayer .NE. temp) conflict = .TRUE.
       CALL getin_p('NOx_RAIN_PULSE', temp)
       IF (ok_pulse_NOx .NE. temp) conflict = .TRUE.
       CALL getin_p('NOx_BBG_FERTIL', temp)
       IF (ok_bbgfertil_NOx .NE. temp) conflict = .TRUE.
       CALL getin_p('NOx_FERTILIZERS_USE', temp)
       IF (ok_cropsfertil_NOx .NE. temp) conflict = .TRUE.
       IF (conflict) THEN
          CALL ipslerr(2,'control_initialize', &
               'Some of the parameter values set in the', &
               'run.def or set as a default are overwritten by the',&
               'values implied by the OK_BVOC flag')
       ENDIF

    ENDIF
    !
    !Config Key   = CHEMISTRY_LEAFAGE
    !Config Desc  = Activate LEAFAGE?
    !Config If    = CHEMISTRY_BVOC
    !Config Def   = n
    !Config Help  = set to TRUE if biogenic emissions calculation takes leaf age into account
    !Config Units = [FLAG]
    CALL getin_p('CHEMISTRY_LEAFAGE', ok_leafage)
    WRITE(numout,*) 'Leaf Age: ', ok_leafage
    
    !Config Key   = CANOPY_EXTINCTION 
    !Config Desc  = Use canopy radiative transfer model?
    !Config If    = CHEMISTRY_BVOC 
    !Config Def   = n
    !Config Help  = set to TRUE if canopy radiative transfer model is used for biogenic emissions 
    !Config Units = [FLAG]
    CALL getin_p('CANOPY_EXTINCTION', ok_radcanopy)
    WRITE(numout,*) 'Canopy radiative transfer model: ', ok_radcanopy
    
    !Config Key   = CANOPY_MULTILAYER
    !Config Desc  = Use canopy radiative transfer model with multi-layers
    !Config If    = CANOPY_EXTINCTION 
    !Config Def   = n
    !Config Help  = set to TRUE if canopy radiative transfer model is with 
    !Config         multiple layers. DO NOT CONFUSE with the settings for the
    !Config         multi-layer energy budget.  
    !Config Units = [FLAG]
    CALL getin_p('CANOPY_MULTILAYER', ok_multilayer)
    WRITE(numout,*) 'Multi-layer Canopy model: ', ok_multilayer
    
    !Config Key   = NOx_RAIN_PULSE
    !Config Desc  = Calculate NOx emissions with pulse?
    !Config If    = CHEMISTRY_BVOC 
    !Config Def   = n
    !Config Help  = set to TRUE if NOx rain pulse is taken into account
    !Config Units = [FLAG]
    CALL getin_p('NOx_RAIN_PULSE', ok_pulse_NOx)
    WRITE(numout,*) 'Rain NOx pulsing: ', ok_pulse_NOx
    
    !Config Key   = NOx_BBG_FERTIL
    !Config Desc  = Calculate NOx emissions with bbg fertilizing effect?
    !Config If    = CHEMISTRY_BVOC 
    !Config Def   = n
    !Config Help  = set to TRUE if NOx emissions are calculated with bbg effect 
    !Config         Fertil effect of bbg on NOx soil emissions 
    !Config Units = [FLAG]
    CALL getin_p('NOx_BBG_FERTIL', ok_bbgfertil_NOx)
    WRITE(numout,*) 'NOx bbg fertil effect: ', ok_bbgfertil_NOx
    
    !Config Key   = NOx_FERTILIZERS_USE
    !Config Desc  = Calculate NOx emissions with fertilizers use?
    !Config If    = CHEMISTRY_BVOC 
    !Config Def   = n
    !Config Help  = set to TRUE if NOx emissions are calculated with fertilizers use
    !Config         Fertilizers use effect on NOx soil emissions  
    !Config Units = [FLAG] 
    CALL getin_p('NOx_FERTILIZERS_USE', ok_cropsfertil_NOx)
    WRITE(numout,*) 'NOx Fertilizers use: ', ok_cropsfertil_NOx
    
    !Config Key  = Is CO2 impact on BVOC accounted for using Possell 2005 ?
    !Config Desc = In this case we use Possell 2005 parameterisation 
    !Config Desc = to take into account the impact of CO2 on biogenic emissions for 
    !Config Desc = isoprene 
    !Config Def  = n 
    !Config Help = set to TRUE if Possell parameterisation has to be considered for the CO2 impact
    !
    ok_co2bvoc_poss = .FALSE.
    CALL getin_p('CO2_FOR_BVOC_POSSELL', ok_co2bvoc_poss)
    WRITE(*,*) 'CO2 impact on BVOC - Possell parameterisation: ', ok_co2bvoc_poss
    !
    !Config Key  = Is CO2 impact on BVOC accounted for using Wilkinson 2009 ? 
    !Config Desc = In this case we use Wilkinson 2009 parameterisation 
    !Config Desc = to take into account the impact of CO2 on biogenic emissions for 
    !Config Desc = isoprene 
    !Config Def  = n 
    !Config Help = set to TRUE if Wilkinson parameterisation has to be considered for the CO2 impact
    !
    ok_co2bvoc_wilk = .FALSE.
    CALL getin_p('CO2_FOR_BVOC_WILKINSON', ok_co2bvoc_wilk)
    WRITE(*,*) 'CO2 impact on BVOC - Wilkinson parameterisation: ', ok_co2bvoc_wilk
    !
    !+++CHECK+++
    ! No longer in CN but still in CN.CAN
    !Config Key  = CONSTANT_MORTALITY
    !Config Desc = Assume constant prescribed mortality or calculate &
    !              mortality as a function of last year's NPP
    !Config If   = OK_STOMATE
    !Config Def  = y
    !Config Help = set to TRUE if constant mortality is to be assumed
    ok_constant_mortality = .TRUE.
    CALL getin_p('CONSTANT_MORTALITY',ok_constant_mortality)
    WRITE(numout,*) 'MORTALITY is assumed to be constant ',&
         '(instead of being a function of vigor): ', &
         ok_constant_mortality
    !+++++++++++
    
    ! Changes following forest management
    !
    !Config Key   = OK_READ_FM_MAP
    !Config Desc  = We need to have the option to read the forest 
    !               management strategy from a map (NetCDF file).  
    !               If this option is equal to TRUE, we will overwrite 
    !               the forest_managed_forced option, so you should be 
    !               careful to only use one or the other.  
    !Config If    = OK_STOMATE
    !Config Def   = N
    !Config Help  =
    !Config Units =
    ok_read_fm_map=.FALSE.
    CALL getin_p('OK_READ_FM_MAP',ok_read_fm_map)
    
    !Config Key   = OK_SPECIES_CHANGE
    !Config Desc  = Sometimes it's a good idea to change species 
    !               after a clearcut on managed forest.  If this 
    !               flag is true, we do this. If not, the same PFT 
    !               is always replanted.
    !Config If    = OK_STOMATE
    !Config Def   = N
    !Config Help  =
    !Config Units =
    ok_change_species=.FALSE.
    CALL getin_p('OK_CHANGE_SPECIES',ok_change_species)
    
    !Config Key   = READ_SPECIES_CHANGE_MAP
    !Config Desc  = If we change the species after a clearcut, do we 
    !               want to read the new PFT from a map?
    !Config If    = OK_STOMATE
    !Config Def   = N
    !Config Help  =
    !Config Units =
    ok_read_species_change_map=.FALSE.
    CALL getin_p('OK_READ_SPECIES_CHANGE_MAP',ok_read_species_change_map)
    
    !Config Key   = OK_READ_DESIRED_FM_MAP
    !Config Desc  = If we change the species after a clearcut, do 
    !               we want to read the new FM strategy from a map?
    !Config If    = OK_STOMATE, OK_CHANGE_SPECIES
    !Config Def   = N
    !Config Help  =
    !Config Units =
    ok_read_desired_fm_map=.FALSE.
    CALL getin_p('OK_READ_DESIRED_FM_MAP',ok_read_desired_fm_map)
    
    !Config Key   = OK_LITTER_RAKING
    !Config Desc  = Check to see if we are interested in using a litter 
    !               demand map to remove litter from forest PFTs and put 
    !               it into agricultural PFTs. This simulations the 
    !               practice of litter raking.
    !Config If    = OK_STOMATE
    !Config Def   = N
    !Config Help  =
    !Config Units =
    ok_litter_raking=.FALSE.
    CALL getin_p('LITTER_RAKING',ok_litter_raking)
       
    !Config Key   = OK_DIMENSIONAL_PRODUCT_USE
    !Config Desc  = Once the wood is harvested (through management or
    !               LCC) it ends up in wood product pools. Two options were 
    !               implemeted: (1) the product use and thus its on longevity,
    !               depends on the dimensions of the harvest. (2) the 
    !               dimensions are ignored and the wood is used according
    !               to fixed ratios.
    !Config If    = OK_STOMATE
    !Config Def   = N
    !Config Help  =
    !Config Units = 
    ok_dimensional_product_use=.TRUE.
    CALL getin_p('DIMENSIONAL_PRODUCT_USE',ok_dimensional_product_use)

    ! Check for consistency
    IF (ok_change_species .AND. ok_litter_raking)THEN

       ! There is no obvious conflict in combining species change
       ! and litter raking but it was never tested. Better not to
       ! combine these settings
       WRITE(numout,*) 'ERROR - conflicting settings in run.def'
       WRITE(numout,*) 'Trying to jointly use litter raking and species change'
       WRITE(numout,*) 'This should be tested first'
       CALL ipslerr(3,'ERROR: run.def',&
            'ERROR - Trying to use litter raking',&
            'at the same time of species change',&
            'The code was not developed or tested for this combination')       
    ENDIF

    !+++CHECK+++
    ! veget_update is not known as this point. How can we test whether 
    ! land cover change is activated? Alternatively we could move this
    ! test but this is the best place to have it.
!!$    IF (ok_change_species .AND. veget_update.GT.zero)THEN
!!$
!!$       ! There is no obvious conflict in combining species change
!!$       ! and land cover change but it was never tested. Better not to
!!$       ! combine these settings
!!$       WRITE(numout,*) 'control.f90 - conflicting settings in stomate_lpj'
!!$       WRITE(numout,*) 'Trying to jointly use land use and species change'
!!$       WRITE(numout,*) 'This should be tested first'
!!$       CALL ipslerr(3,'ERROR: run.def',&
!!$            'ERROR - Trying to use land use ',&
!!$            'at the same time of species change',&
!!$            'The code was not loped or tested for this combination')
!!$    ENDIF
    !++++++++++

    WRITE(numout,*) 'OK_HYDROL_ARCH is activated, ',ok_hydrol_arch
    WRITE(numout,*) 'OK_GS_FEEDBACK is activated, ',ok_gs_feedback
    IF (ok_hydrol_arch .AND. ok_gs_feedback)THEN

       ! This is the default set-up for running hydraulic architecture 
       ! in production mode. Using full functionality
       WRITE(numout,*) 'Hydraulic architecture, stomatal feedback'
       WRITE(numout,*) 'and the energy budget are (re)calculated'

    ELSEIF (.NOT. ok_hydrol_arch .AND. ok_gs_feedback) THEN

       ! The debug options should not be used. Not clear whether
       ! the user want to use hydraulic architecture or not
       WRITE(numout,*) 'Hydraulic architecture is not used'
       WRITE(numout,*) 'but the debug option was set to TRUE'
       WRITE(numout,*) 'Not clear whether you want to use '
       WRITE(numout,*) 'hydraulic architecture or not'
       CALL ipslerr(3,'control.f90 - run.def',&
            'setting for stomatal feedback was in conflict with',&
            'the setting of hydraulic architecture','')

    ELSEIF (ok_hydrol_arch .AND. .NOT.ok_gs_feedback) THEN

       ! Warning
       CALL ipslerr(2,'control.f90 - hydraulic architecture',&
            'is run in DEBUG mode',&
            'stomatal feedback is not used','')

    ENDIF

    WRITE(numout,*) 'OK_MLEB is activated, ',ok_mleb
    WRITE(numout,*) 'OK_IMPOSE_CAN_STRUCTURE is activated, ',ok_impose_can_structure
    WRITE(numout,*) 'OK_MLEB_HISTORY_FILE is activated, ',ok_mleb_history_file
    IF (ok_mleb .AND. ok_impose_can_structure) THEN

       ! Warning
       CALL ipslerr(2,'control.f90 - multi-layer energy budget',&
            'is run in debug mode',&
            'The canopy structure is fixed and imposed','')

    ELSEIF (ok_mleb .AND. .NOT.ok_mleb_history_file) THEN
    
       ! Warning
       CALL ipslerr(2,'control.f90 - multi-layer energy budget',&
            'There won be any output files for the multi-layer energy budget',&
            'If this is not intended, change the run.def','')

    ELSEIF ((.NOT.ok_mleb .AND. ok_impose_can_structure) .OR. &
         (.NOT.ok_mleb .AND. ok_mleb_history_file)) THEN
       
       ! The debug options should not be used. Not clear whether
       ! the user want to use hydraulic architecture or not
       WRITE(numout,*) 'The multi-layer energy budget is not used'
       WRITE(numout,*) 'Debug options are set to FALSE'
       CALL ipslerr(3,'control.f90 - run.def',&
            'inconsistent settings of de debug options',&
            'because the multi-level enery budget itself is not used','')

    ENDIF
    
    IF ( ok_dgvm ) THEN

       ok_stomate = .TRUE.
       CALL ipslerr(2,'control_initialize', &
            'If you want to use the DGVM you need STOMATE',&
            'activating the flag OK_STOMATE',&
            'We set OK_STOMATE to TRUE to ensure consistency')
    ENDIF

    IF ( ok_multilayer .AND. .NOT.(ok_radcanopy) ) THEN

       ok_radcanopy  = .TRUE.
       CALL ipslerr(2,'control_initialize', &
               'You want to use the multilayer model without ', &
               'activating the flag CANOPY_EXTINCTION',&
               'We set CANOPY_EXTINCTION to TRUE to ensure consistency')
    ENDIF
    !
    !Config Key   = OK_C13
    !Config Desc  = Calculate C13 fractionation
    !Config If    = OK_SECHIBA 
    !Config Def   = n
    !Config Help  = set to TRUE if C13 is to be calculated
    !Config Units = [FLAG]
    ok_c13 = .FALSE.
    CALL getin_p('OK_C13', ok_c13)
    WRITE(numout,*) 'C13 fractionation is calculated: ', ok_c13
    ! 
    !Config Key  = RECRUITMENT 
    !Config Desc = Activate recruitment. The default is no recruitment 
    !Config Def  = n 
    !Config Help = Set to TRUE if recruitment is activated 
    !Config Units = [FLAG]  
    ok_recruitment = .FALSE. 
    CALL getin_p('OK_RECRUITMENT',ok_recruitment) 
    WRITE(numout,*) 'Recruitment is activated: ',ok_recruitment 

    ok_windthrow = .FALSE.
    CALL getin_p('OK_WINDTHROW',ok_windthrow) 
    WRITE(numout,*) 'Windthrow is activated: ',ok_windthrow

    !
    ! Here we need the same initialisation as above
    !
    ok_pheno = .TRUE.

    !
    ! Configuration : number of PFTs and parameters
    !

    ! 1. Number of PFTs defined by the user

    !Config Key   = NVM
    !Config Desc  = number of PFTs  
    !Config If    = OK_SECHIBA or OK_STOMATE
    !Config Def   = 13
    !Config Help  = The number of vegetation types define by the user
    !Config Units = [-]
    !
    CALL getin_p('NVM',nvm)
    WRITE(numout,*)'the number of pfts used by the model is : ', nvm

    ! 2. Should we read the parameters in the run.def file ?

    !Config Key   = IMPOSE_PARAM
    !Config Desc  = Do you impose the values of the parameters?
    !Config if    = OK_SECHIBA or OK_STOMATE
    !Config Def   = y
    !Config Help  = This flag can deactivate the reading of some parameters.
    !               Useful if you want to use the standard values without commenting 
    !               the run.def
    !Config Units = [FLAG]
    !
    CALL getin_p('IMPOSE_PARAM',impose_param)


    !! Initialize vertical discretization
    !! All initialization is done in the vertical module
    !! Calculate ngrnd and nslm
    CALL vertical_soil_init
    
    
    ! 3. Allocate and intialize the pft parameters

    CALL pft_parameters_main()

    ! 4. Activation sub-models of ORCHIDEE

    CALL activate_sub_models()

    ! 5. Vegetation configuration

    CALL veget_config

    ! 6. Read the parameters in the run.def file  according the flags

    IF (impose_param ) THEN 
       CALL config_pft_parameters
    ENDIF

    IF ( ok_sechiba ) THEN
       IF (impose_param ) THEN
          IF (printlev>=2) WRITE(numout,*)'In control_initialize: &
              & call config_sechiba_parameters and config_sechiba_pft_parameters' 
          CALL config_sechiba_parameters
          CALL config_sechiba_pft_parameters()
       ENDIF
    ENDIF


    ! Initialize variables in constantes_soil
    CALL config_soil_parameters()


    ! Coherence check for depth of thermosoil for long term simulation where 
    ! soil thermal inertia matters
    ! ok_freeze_thermix is defined in config_soil_parameters
    IF (ok_freeze_thermix .AND. zmaxt < 11) THEN
       WRITE(numout,*) 'ERROR : Incoherence between ok_freeze_thermix activated &
            and soil depth too small. '
       WRITE(numout,*) 'Here a soil depth of ', zmaxt, 'm is used for the soil &
            thermodynamics'
       WRITE(numout,*) 'Set DEPTH_MAX_T=11 or higher in run.def parameter file or &
            deactivate soil freezing'
       CALL ipslerr(3,'control_initialize','Too shallow soil chosen for the &
            thermodynamic for soil freezing', &
            'Adapt run.def with at least DEPTH_MAX=11','')
    END IF
        
    ! Define diaglev
    ! We take the top nslm (number of layer in CWRR) layer of the thermodynamics 
    ! for the diagnostics. The layers in the hydrology and the thermodynamics are
    ! placed a the same depth (the top nslm layers) but the upper boundary condition
    ! is simpler in the thermodynamics. 
    nbdl = nslm
    ALLOCATE(diaglev(nbdl), stat=ier)
    IF (ier /= 0) CALL ipslerr(3,'control_initialize','Pb in allocation of diaglev','','')

    ! Get diaglev from module vertical for CWRR
    diaglev=znt(1:nbdl)
    IF (printlev>=2) WRITE(numout,*) 'In control_initialize, diaglev = ',diaglev

    IF ( impose_param ) THEN
       IF (printlev>=2) WRITE(numout,*)'In control_initialize: call &
            & config_co2_parameters'
       CALL config_co2_parameters
    ENDIF
    
    IF ( ok_stomate ) THEN
       IF ( impose_param ) THEN
          IF (printlev>=2) WRITE(numout,*)'In control_initialize: &
              & call config_stomate_parameters'
          CALL config_stomate_parameters
          CALL config_stomate_pft_parameters
       ENDIF
    ENDIF
    
    IF ( ok_dgvm ) THEN
       IF ( impose_param ) THEN
          IF (printlev>=2) WRITE(numout,*)'In control_initialize: &
             & call config_dgvm_parameters'
          CALL config_dgvm_parameters
       ENDIF
    ENDIF    
  END SUBROUTINE control_initialize
  
END MODULE control
