! =================================================================================================================================
! MODULE       : sapiens_product_use
!
! CONTACT      : orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF       Following lateral transport, calculate C-stored in the product pools
!!             and the CO2-release from product decomposition  
!!
!!\n DESCRIPTION: None
!!
!! RECENT CHANGE(S): None
!!
!! REFERENCE(S)	: None
!!
!! SVN          :
!! $HeadURL: svn://forge.ipsl.jussieu.fr/orchidee/branches/ORCHIDEE-DOFOCO/ORCHIDEE/src_stomate/stomate_lcchange.f90 $
!! $Date: 2013-07-03 10:16:15 +0200 (Wed, 03 Jul 2013) $
!! $Revision: 1356 $
!! \n
!_ ================================================================================================================================

MODULE sapiens_product_use

! modules used:
  
  USE stomate_data
  USE pft_parameters
  USE constantes
  USE function_library,    ONLY: check_mass_balance
  
  IMPLICIT NONE
  
  PRIVATE
  PUBLIC basic_product_use, product_init, dim_product_use

CONTAINS 


!! ================================================================================================================================
!! SUBROUTINE   : basic_product_use
!!
!>\BRIEF        Calculate the C stored in biomass-based products and the 
!!              CO2 release through their decomposition 
!!
!! DESCRIPTION  : The basic approach distinguished 3 pools (as introduced by
!! Shilong Piao based on Houghton. This module does NOT make use of the 
!! dimensions of the woody harvest or of the management and cutting flags.
!! All wood is distributed across the short, medium and long product pools. 
!! This basically means that wood use in ..., 1600, 1700, 1800, 1900 and 2000 
!! was identical.\n
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S) : prod_s, prod_m, prod_l, flux_s, flux_m, flux_l,
!!   flux_prod_s flux_prod_m and flux_prod1_l 
!!
!! REFERENCES   : None
!!
!! FLOWCHART    : None
!!
!_ ================================================================================================================================

 SUBROUTINE basic_product_use(npts, dt_days, harvest_pool_bound, harvest_pool, &
            harvest_type, harvest_cut, harvest_area, &
            flux_prod_s, flux_prod_m, flux_prod_l, prod_s, &
            prod_m, prod_l, prod_s_total, prod_m_total, &
            prod_l_total, flux_prod_total, flux_s, flux_m, &
            flux_l, resolution, veget_cov_max)

   IMPLICIT NONE


 !! 0. Variable and parameter declaration 
    
    !! 0.1 Input variables
   
    INTEGER, INTENT(in)                                  :: npts              !! Domain size - number of pixels (unitless)
    REAL(r_std), INTENT(in)                              :: dt_days           !! Time step of vegetation dynamics for stomate
                                                                              !! (days)
    REAL(r_std),DIMENSION(:,:), INTENT(in)               :: resolution        !! The length in m of each grid-box in X and Y
    REAL(r_std), DIMENSION(:), INTENT(in)                :: harvest_pool_bound!! The boundaries of the diameter classes
                                                                              !! in the wood harvest pools @tex $(m)$ @endtex
     REAL(r_std), DIMENSION(:,:), INTENT(in)             :: veget_cov_max         !! Passed so we can check_mass_balance 
                                                                              !! Making it OPTIONAL would have been nicer.


    !! 0.2 Output variables

    REAL(r_std), DIMENSION(:,:), INTENT(out)             :: flux_prod_s       !! C-released during first years (short term) 
                                                                              !! following land cover change 
                                                                              !! @tex ($gC m^{-1} dt^{-1}$) @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(out)             :: flux_prod_m       !! Total annual release from decomposition of 
                                                                              !! the medium-lived product pool 
                                                                              !! @tex ($gC m^{-1} dt^{-1}$) @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(out)             :: flux_prod_l       !! Total annual release from decomposition of 
                                                                              !! the long-lived product pool 
                                                                              !! @tex ($gC m^{-1} dt^{-1}$) @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(out)             :: prod_s_total      !! Total products remaining in the short-lived  
                                                                              !! pool after the annual decomposition 
                                                                              !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(out)             :: prod_m_total      !! Total products remaining in the medium-lived
                                                                              !! pool after the annual decomposition 
                                                                              !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(out)             :: prod_l_total      !! Total products remaining in the long-lived
                                                                              !! pool after the annual release 
                                                                              !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(out)             :: flux_prod_total   !! Total flux from decomposition of the short,
                                                                              !! medium and long lived product pools 
                                                                              !! @tex $(gC m^{-1} dt^{-1})$ @endtex

    !! 0.3 Modified variables

    REAL(r_std), DIMENSION(:,0:,:), INTENT(inout)        :: prod_s            !! Short-lived product pool after the annual 
                                                                              !! release of each compartment (short + 1 : 
                                                                              !! input from year of land cover change) 
                                                                              !! @tex ($gC pixel^{-1}$) @endtex    
    REAL(r_std), DIMENSION(:,0:,:), INTENT(inout)        :: prod_m            !! Medium-lived product pool after the annual 
                                                                              !! release of each compartment (medium + 1 : 
                                                                              !! input from year of land cover change) 
                                                                              !! @tex ($gC pixel^{-1}$) @endtex 
    REAL(r_std), DIMENSION(:,0:,:), INTENT(inout)        :: prod_l            !! long-lived product pool after the annual 
                                                                              !! release of each compartment (long + 1 : 
                                                                              !! input from year of land cover change) 
                                                                              !! @tex ($gC pixel^{-1}$) @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)         :: flux_s            !! Annual release from the short-lived product 
                                                                              !! pool @tex $(gC) pixel^{-1} year^{-1}$ @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)         :: flux_m            !! Annual release from the medium-lived product  
                                                                              !! pool @tex $(gC) pixel^{-1} year^{-1}$ @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)         :: flux_l            !! Annual release from the long-lived product
                                                                              !! pool @tex $(gC) pixel^{-1} year^{-1}$ @endtex
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)       :: harvest_pool      !! The wood and biomass that have been
                                                                              !! havested by humans @tex $(gC)$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)           :: harvest_type      !! Type of management that resulted
                                                                              !! in the harvest (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)           :: harvest_cut       !! Type of cutting that was used for the harvest
                                                                              !! (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)           :: harvest_area      !! Harvested area (m^{2})
   

    !! 0.4 Local variables
 
    INTEGER(i_std)                                       :: ipts, ivm, iage   !! Indices (unitless)
    INTEGER(i_std)                                       :: imbc, iele        !! Indices (unitless)
    REAL(r_std), DIMENSION(nelements)                    :: exported_biomass  !! Total biomass exported from the PFT (gC)
    REAL(r_std), DIMENSION(nelements)                    :: residual          !! Residual @tex $(gC m^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,nmbcomp,nelements)   :: check_intern      !! Contains the components of the internal
                                                                              !! mass balance chech for this routine
                                                                              !! @tex $(gC m^{-2} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)           :: closure_intern    !! Check closure of internal mass balance
                                                                              !! @tex $(gC m^{-2} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)           :: pool_start        !! Start pool of this routine 
                                                                              !! @tex $(gC m^{-2} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)           :: pool_end          !! End pool of this routine 
                                                                              !! @tex $(gC m^{-2} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts)                         :: pixel_area        !! surface area of the pixel @tex ($m^{2}$) @endtex
    
!_ ================================================================================================================================    

    IF(printlev.GE.3) WRITE(numout,*) 'Entering basic product use'
 

 !! 1. initialization

    !! 1.1 Fluxes and pools
    prod_s(:,0,:) = zero
    prod_m(:,0,:) = zero
    prod_l(:,0,:) = zero
    prod_s_total(:,:) = zero
    prod_m_total(:,:) = zero
    prod_l_total(:,:) = zero   
    flux_prod_s(:,:) = zero
    flux_prod_m(:,:) = zero
    flux_prod_l(:,:) = zero
    flux_prod_total(:,:) = zero

    !! 1.2 Pixel area
    !  Surface area (m2) of the pixel
    pixel_area(:) = resolution(:,1) * resolution(:,2)

    !! 1.3 Initialize local printlev
    !printlev_loc=get_printlev('product_use')

    !! 1.4 Initialize check for mass balance closure 
    IF (err_act.GT.1) THEN

       ! The biomass harvest pool shouldn't be multiplied by veget_cov_max
       ! its units are already in gC pixel-1. The total amount for the 
       ! pixel was stored in harvest_pool. Account for the harvest and
       ! wood product pools. There are no longer PFTs
       check_intern(:,:,:,:) = zero
       pool_start(:,:,:) = zero
       DO iele = 1,nelements
          pool_start(:,1,iele) = &
               ( SUM(SUM(harvest_pool(:,:,:,iele),3),2) + &
               SUM(prod_l(:,:,iele),2) + SUM(prod_m(:,:,iele),2) + &
               SUM(prod_s(:,:,iele),2) )/pixel_area(:)
       ENDDO
       
    ENDIF ! err_act.GT.1

    !! 1.5 Logical check of coeff_lcchange
    !  Within a PFT the coeff_lcchange should add up to one. Else the 
    !  carbon balance will not be closed! Do not include PFT 1 in this 
    !  check because its coeff_lcchange were set to undef (-9999).
    DO ivm = 2,nvm

       IF ( (coeff_lcchange_s(ivm) + coeff_lcchange_m(ivm) + &
            coeff_lcchange_l(ivm) .LT. un - min_stomate) .OR. &
            (coeff_lcchange_s(ivm) + coeff_lcchange_m(ivm) + &
            coeff_lcchange_l(ivm) .GT. un + min_stomate) ) THEN

          WRITE(numout,*) 'Error: coeff_lcchange in sapiens_product_use.f90 ' //&
               'do not add up to 1 in PFT - cannot close the mass balance, ',ivm
          CALL ipslerr_p (3,'sapiens_product_use', &
               'basic_product_use','coeff_lcchange do not add up to 1 ','')
       ENDIF

    ENDDO


 !! 2. Partition harvest in a short, medium and long-lived product pool

    DO ipts = 1,npts

       !! 2.1 Calculate the initial mass of the product pool (gC)
       DO ivm = 2,nvm
  
          !+++NOTE+++
          ! For the moment all harvest is lumped together and we don't 
          ! make use of harvest_type and harvest_cut to refine the product
          ! use. If these variables are to be used be aware that for the
          ! moment these values can be overwritten within a simulation.
          ! Although the sequence of the calls in stomate_lpj is believed
          ! to protect against this, it is not enforced by the code. For
          ! example, if we first thin and then harvest, wood of both
          ! actions will be stored in the harvest pool but it will be 
          ! recored as just harvest (thinning will be overwritten by harvest)
          exported_biomass(:) = SUM(harvest_pool(ipts,ivm,:,:),1)
          !+++++++++++

          ! Attribute the harvested biomass to pools with a different 
          ! turnover time. The turnover times are PFT depend which 
          ! accounts for the different uses different PFT's are put to i.e. 
          ! wood from forests vs grass or crops. The values stored in 
          ! harvest_pool are absolute numbers gC (for that pixel), hence, 
          ! the spatial extent of the LCC change, harvest or thinning has 
          ! already been accounted for.
          prod_s(ipts,0,:) = prod_s(ipts,0,:) + &
               (coeff_lcchange_s(ivm) * exported_biomass(:))
          prod_m(ipts,0,:) = prod_m(ipts,0,:) + &
               (coeff_lcchange_m(ivm) * exported_biomass(:))
          prod_l(ipts,0,:) = prod_l(ipts,0,:) + &
               (coeff_lcchange_l(ivm) * exported_biomass(:))

       ENDDO

       !! 2.2 Check whether all the harvest was used as is to be expected
       !  The precision we are aiming for is 10-8 per m-2. The harvest pool
       !  is expressed for the whole pixel
       residual(:) = SUM(SUM(harvest_pool(ipts,:,:,:),2),1) - &
            prod_s(ipts,0,:) - prod_m(ipts,0,:) - &
            prod_l(ipts,0,:)
       
       DO iele = 1,nelements

          IF (residual(iele)/pixel_area(ipts) .GT. min_stomate .OR. &
               residual(iele)/pixel_area(ipts) .LT. - min_stomate) THEN
             
             WRITE (numout,*) 'Error: some harvest in '//&
                  'sapiens_product_use has not been attributed'
             WRITE (numout,*) 'Residual in pixel, iele, ', &
                  iele, ipts, residual
             CALL ipslerr_p (3,'sapiens_product_use', &
                  'basic_product_use','some harvest has not been attributed',&
                  '')

          ELSE

             ! We could still have residual within the precision at the 
             ! m2 level but outside the precision at the pixel level. 
             ! Move the residual to the short lived pool
             prod_s(ipts,0,iele) = prod_s(ipts,0,iele) + residual(iele)

             ! All the carbon in the harvest pool was allocated to the product 
             ! pools the harvest pool can be set to zero so we can us it with 
             ! confidence in the mass balance calculation
             harvest_pool(ipts,:,:,iele) = zero
             harvest_type(ipts,:) = zero
             harvest_cut(ipts,:) = zero
             harvest_area(ipts,:) = zero

          ENDIF 

       ENDDO
    
       !! 2.3 Update the long-turnover pool content
       !  Update the long-turnover pool content following flux emission
       !  of a linear decay (1/longivety) of the initial carbon input.
       !  This must be implemented with a counter going from ::nlong to 
       !  zero because this way it takes ::nlong years before the intial 
       !  pool has been entirely consumed. If a more intuitive approach 
       !  would have been used in which the counter goes from zero to
       !  ::nlong, the pool would be depleted in the first year.
       !  Update the long-turnover pool content following flux emission.
       DO iage = nlong,2,-1

          ! flux and prod are large numbers. At the last time step 10-5
          ! gC remains which is outside the precision. Therefore, this 
          ! carbon is moved to the flux 
          IF (iage .EQ. nlong) THEN
              
              flux_prod_l(ipts,:) = flux_prod_l(ipts,:) + prod_l(ipts,iage-1,:)
              flux_l(ipts,iage,:) = prod_l(ipts,iage-1,:)
              prod_l(ipts,iage,:) = zero

          ELSE
          
             ! ::flux_l has INTENT(inout), it thus has the value from the 
             ! previuous year at the start of these calculations. Copy
             ! the flux from the previous year to this year and update
             ! the product pool
             flux_prod_l(ipts,:) = flux_prod_l(ipts,:) + flux_l(ipts,iage,:)
             prod_l(ipts,iage,:) = prod_l(ipts,iage-1,:) - flux_l(ipts,iage-1,:)
             flux_l(ipts,iage,:) = flux_l(ipts,iage-1,:)
 
          ENDIF
          
       ENDDO

       ! Treat the first year separately because it is not included in 
       ! the loop above
       flux_prod_l(ipts,:) = flux_prod_l(ipts,:) + flux_l(ipts,1,:)

       ! Each year 1/::nlong of the initial pool is decomposed. This fixed
       ! amount of decomposition is moved to a higher age class each year. 
       ! The units of ::flux_m are gC (absolute number as is the product 
       ! pool). At this point in the calculation, the flux is NOT a flux 
       ! yet but is also a pool 
       flux_l(ipts,1,:) = (un/nlong) * prod_l(ipts,0,:)
       prod_l(ipts,1,:) = prod_l(ipts,0,:) - flux_l(ipts,1,:)
       prod_l(ipts,0,:) = zero

       !! 2.4 Update the medium-turnover pool content 
       !  Update the medium-turnover pool content following flux emission.
       !  For comments see the section above. The concept is identical. 
       DO iage = nmedium,2,-1

          IF (iage .EQ. nmedium) THEN
              
              flux_prod_m(ipts,:) = flux_prod_m(ipts,:) + prod_m(ipts,iage-1,:)
              flux_m(ipts,iage,:) = prod_m(ipts,iage-1,:)
              prod_m(ipts,iage,:) = zero

          ELSE

             flux_prod_m(ipts,:) = flux_prod_m(ipts,:) + flux_m(ipts,iage,:)
             prod_m(ipts,iage,:) = prod_m(ipts,iage-1,:) - flux_m(ipts,iage-1,:)
             flux_m(ipts,iage,:) = flux_m(ipts,iage-1,:)
         
          ENDIF

       ENDDO
     
       flux_prod_m(ipts,:) = flux_prod_m(ipts,:) + flux_m(ipts,1,:)
       flux_m(ipts,1,:) = (un/nmedium) * prod_m(ipts,0,:)
       prod_m(ipts,1,:) = prod_m(ipts,0,:) - flux_m(ipts,1,:)
       prod_m(ipts,0,:) = zero

       !! 2.5 Update the short-turnover pool
       !  The length of the short-turnover pool will most likely be
       !  less than length of the regular loop. The regular loop is
       !  now conditional. 
       !  NOTE: this code has only been tested for nshort = 1 and 
       !  nshort = 2
       IF (nshort .GE. 4) THEN

          DO iage = nshort,2,-1

             IF (iage .EQ. nshort) THEN
              
                flux_prod_s(ipts,:) = flux_prod_s(ipts,:) + &
                     prod_s(ipts,iage-1,:)
                flux_s(ipts,iage,:) = prod_s(ipts,iage-1,:)
                prod_s(ipts,iage,:) = zero

             ELSE
  
                flux_prod_s(ipts,:) = flux_prod_s(ipts,:) + &
                     flux_s(ipts,iage,:)
                prod_s(ipts,iage,:) = prod_s(ipts,iage-1,:) - &
                     flux_s(ipts,iage-1,:)
                flux_s(ipts,iage,:) = flux_s(ipts,iage-1,:)

             ENDIF

          ENDDO

          flux_prod_s(ipts,:) = flux_prod_s(ipts,:) + flux_s(ipts,1,:) 
          flux_s(ipts,1,:) = (un/nshort) * prod_s(ipts,0,:)
          prod_s(ipts,1,:) = prod_s(ipts,0,:) - flux_s(ipts,1,:)
          prod_s(ipts,0,:) = zero

       ELSEIF (nshort .EQ. 3) THEN

          ! Decompose the product pool from the last year
          flux_prod_s(ipts,:) = flux_prod_s(ipts,:) + prod_s(ipts,2,:)
          flux_s(ipts,3,:) = prod_s(ipts,2,:)
          prod_s(ipts,3,:) = zero
          flux_prod_s(ipts,:) = flux_prod_s(ipts,:) + flux_s(ipts,2,:)
          prod_s(ipts,2,:) = prod_s(ipts,1,:) - flux_s(ipts,1,:)
          flux_s(ipts,2,:) = flux_s(ipts,1,:)

          ! Calculate the linear product decomposition for this
          ! year's harvest
          flux_prod_s(ipts,:) = flux_prod_s(ipts,:) + flux_s(ipts,1,:) 
          flux_s(ipts,1,:) = (un/nshort) * prod_s(ipts,0,:)
          prod_s(ipts,1,:) = prod_s(ipts,0,:) - flux_s(ipts,1,:)
          prod_s(ipts,0,:) = zero

       ELSEIF (nshort .EQ. 2) THEN

          ! Decompose the product pool from last year
          flux_prod_s(ipts,:) = flux_prod_s(ipts,:) + prod_s(ipts,1,:)
          flux_s(ipts,2,:) = prod_s(ipts,1,:)
          prod_s(ipts,2,:) = zero

          ! Calculate the linear product decomposition for this
          ! year's harvest
          flux_prod_s(ipts,:) = flux_prod_s(ipts,:) + flux_s(ipts,1,:) 
          flux_s(ipts,1,:) = (un/nshort) * prod_s(ipts,0,:)
          prod_s(ipts,1,:) = prod_s(ipts,0,:) - flux_s(ipts,1,:)
          prod_s(ipts,0,:) = zero

       ELSEIF (nshort .EQ. 1) THEN

          ! Everything decomposes in a single year. The short product
          ! pool remains empty.
          flux_prod_s(ipts,:) = flux_prod_s(ipts,:) + prod_s(ipts,0,:) 
          flux_s(ipts,1,:) = prod_s(ipts,0,:)
          prod_s(ipts,1,:) = zero
          prod_s(ipts,0,:) = zero

       ELSE

          WRITE(numout,*) 'Error: flaw in the logic of IF-construct '//&
               '(basic_product_use in sapiens_product_use.f90)'
          CALL ipslerr_p(3,'ERROR: product_use',&
               'Logic flaw in if-construct','','')
          
       ENDIF ! number of ages in short-turnover product pool

    ENDDO ! #pixels - spatial domain  


 !! 3. Check numerical consistency of this routine

    IF (err_act.GT.1) THEN

       ! 3.1 Mass balance closure 
       ! 3.1.1 Calculate final biomass
       ! The biomass harvest pool shouldn't be multiplied by veget_cov_max
       ! its in gC pixel-1 so divide by pixel_area to get a value in
       ! gC m-2. Account for the harvest and wood product pools. The harvest 
       ! pool should be empty, it was added for completeness.
       pool_end = zero
       DO iele=1,nelements
          pool_end(:,1,iele) = &
               ( SUM(SUM(harvest_pool(:,:,:,iele),3),2) + &
               SUM(prod_l(:,:,iele),2) + SUM(prod_m(:,:,iele),2) + &
               SUM(prod_s(:,:,iele),2) ) / pixel_area(:)
       ENDDO

       !! 3.1.2 Calculate mass balance
       ! Common processes
       DO iele=1,nelements
          check_intern(:,1,iland2atm,iele) = &
               -un * ( SUM(flux_s(:,:,iele),2) + &
               SUM(flux_m(:,:,iele),2) + SUM(flux_l(:,:,iele),2) ) / &
               pixel_area(:)
          check_intern(:,1,ipoolchange,iele) = &
               -un * (pool_end(:,1,iele) - &
               pool_start(:,1,iele))
       ENDDO

       closure_intern = zero
       DO imbc = 1,nmbcomp
          DO iele=1,nelements
             closure_intern(:,1,iele) = closure_intern(:,1,iele) + &
                  check_intern(:,1,imbc,iele)
          ENDDO
       ENDDO

       ! 3.1.3 Check mass balance closure
       CALL check_mass_balance("basic_product_use", closure_intern, npts, &
            pool_end, pool_start, veget_cov_max, 'products')
      
    ENDIF ! err_act.GT.1

 !! 4. Convert the pools into fluxes and aggregate

    !  Convert pools into fluxes
    !  The pools are given in the absolute amount of carbon for the 
    !  whole pixel and the harvest was accumulated over the year. 
    !  The unit is thus gc pixel-1 year-1. These fluxes should be 
    !  expressed as gC m-2 dt-1. Therefore, divide by the resolution 
    !  of the pixel and the number of time steps within a year
    DO iele = 1,nelements
       flux_prod_s(:,iele) = flux_prod_s(:,iele) / one_year * &
            dt_days / pixel_area(:) 
       flux_prod_m(:,iele) = flux_prod_m(:,iele) / one_year * &
            dt_days / pixel_area(:) 
       flux_prod_l(:,iele) = flux_prod_l(:,iele) / one_year * &
            dt_days / pixel_area(:) 

       ! Calculate aggregate values
       ! flux_prod_total is now in gC m-2 dt-1. The prod_x_total 
       ! are in gC m-2 
       flux_prod_total(:,iele) = flux_prod_s(:,iele) + &
            flux_prod_m(:,iele) + flux_prod_l(:,iele)
       prod_s_total(:,iele) = SUM(prod_s(:,:,iele),2) / pixel_area(:)
       prod_m_total(:,iele) = SUM(prod_m(:,:,iele),2) / pixel_area(:) 
       prod_l_total(:,iele) = SUM(prod_l(:,:,iele),2) / pixel_area(:) 
       
    ENDDO

    IF(printlev.GE.4) WRITE(numout,*) 'Leaving basic product use'

  END SUBROUTINE basic_product_use


!! ================================================================================================================================
!! SUBROUTINE   : dim_product_use
!!
!>\BRIEF        Calculate the C stored in biomass-based products and the 
!!              CO2 release through their decomposition. 
!!
!! DESCRIPTION  : The basic approach distinguished 3 pools (as introduced by
!! Shilong Piao based on Houghton. Wood harvest with small dimensions is being 
!! used as fire wood the remaining wood goes into the short, medium and long 
!! product pools. This implies that when the dimensions of the harvest change, 
!! the product pools will change as well.\n
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S) : prod_s, prod_m, prod_l, flux_s, flux_m, flux_l,
!!   flux_prod_s flux_prod_m and flux_prod1_l 
!!
!! REFERENCES   : None
!!
!! FLOWCHART    : None
!!
!_ ================================================================================================================================

 SUBROUTINE dim_product_use(npts, dt_days, harvest_pool_bound, harvest_pool, &
            harvest_type, harvest_cut, harvest_area, &
            flux_prod_s, flux_prod_m, flux_prod_l, prod_s, &
            prod_m, prod_l, prod_s_total, prod_m_total, &
            prod_l_total, flux_prod_total, flux_s, flux_m, &
            flux_l, resolution,veget_cov_max)

   IMPLICIT NONE


 !! 0. Variable and parameter declaration 
    
    !! 0.1 Input variables
   
    INTEGER, INTENT(in)                                  :: npts              !! Domain size - number of pixels (unitless)
    REAL(r_std), INTENT(in)                              :: dt_days           !! Time step of vegetation dynamics for stomate
                                                                              !! (days)
    REAL(r_std),DIMENSION(:,:), INTENT(in)               :: resolution        !! The length in m of each grid-box in X and Y
    REAL(r_std), DIMENSION(:), INTENT(in)                :: harvest_pool_bound!! The boundaries of the diameter classes
                                                                              !! in the wood harvest pools @tex $(m)$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(in)              :: veget_cov_max         !! Passed so we can check_mass_balance 
                                                                              !! Making it OPTIONAL would have been nicer.


 !! 0.2 Output variables

    REAL(r_std), DIMENSION(:,:), INTENT(out)             :: flux_prod_s       !! C-released during first years (short term) 
                                                                              !! following land cover change 
                                                                              !! @tex ($gC m^{-1} dt^{-1}$) @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(out)             :: flux_prod_m       !! Total annual release from decomposition of 
                                                                              !! the medium-lived product pool 
                                                                              !! @tex ($gC m^{-1} dt^{-1}$) @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(out)             :: flux_prod_l       !! Total annual release from decomposition of 
                                                                              !! the long-lived product pool 
                                                                              !! @tex ($gC m^{-1} dt^{-1}$) @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(out)             :: prod_s_total      !! Total products remaining in the short-lived  
                                                                              !! pool after the annual decomposition 
                                                                              !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(out)             :: prod_m_total      !! Total products remaining in the medium-lived
                                                                              !! pool after the annual decomposition 
                                                                              !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(out)             :: prod_l_total      !! Total products remaining in the long-lived
                                                                              !! pool after the annual release 
                                                                              !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(out)             :: flux_prod_total   !! Total flux from decomposition of the short,
                                                                              !! medium and long lived product pools 
                                                                              !! @tex $(gC m^{-1} dt^{-1})$ @endtex

    !! 0.3 Modified variables

    REAL(r_std), DIMENSION(:,0:,:), INTENT(inout)        :: prod_s            !! Short-lived product pool after the annual 
                                                                              !! release of each compartment (short + 1 : 
                                                                              !! input from year of land cover change) 
                                                                              !! @tex ($gC pixel^{-1}$) @endtex    
    REAL(r_std), DIMENSION(:,0:,:), INTENT(inout)        :: prod_m            !! Medium-lived product pool after the annual 
                                                                              !! release of each compartment (medium + 1 : 
                                                                              !! input from year of land cover change) 
                                                                              !! @tex ($gC pixel^{-1}$) @endtex 
    REAL(r_std), DIMENSION(:,0:,:), INTENT(inout)        :: prod_l            !! long-lived product pool after the annual 
                                                                              !! release of each compartment (long + 1 : 
                                                                              !! input from year of land cover change) 
                                                                              !! @tex ($gC pixel^{-1}$) @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)         :: flux_s            !! Annual release from the short-lived product 
                                                                              !! pool @tex $(gC) pixel^{-1} year^{-1}$ @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)         :: flux_m            !! Annual release from the medium-lived product  
                                                                              !! pool @tex $(gC) pixel^{-1} year^{-1}$ @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)         :: flux_l            !! Annual release from the long-lived product
                                                                              !! pool @tex $(gC) pixel^{-1} year^{-1}$ @endtex
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)       :: harvest_pool      !! The wood and biomass that have been
                                                                              !! havested by humans @tex $(gC)$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)           :: harvest_type      !! Type of management that resulted
                                                                              !! in the harvest (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)           :: harvest_cut       !! Type of cutting that was used for the harvest
                                                                              !! (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)           :: harvest_area      !! Harvested area (m^{2})
   

    !! 0.4 Local variables
 
    INTEGER(i_std)                                       :: ipts, ivm, iage   !! Indices (unitless)
    INTEGER(i_std)                                       :: imbc, idia, iele  !! Indices (unitless)
    REAL(r_std), DIMENSION(nelements)                    :: exported_biomass  !! Total biomass exported from the PFT (gC)
    REAL(r_std), DIMENSION(nelements)                    :: fuelwood_biomass  !! Harvest with small dimensions that will
                                                                              !! be used as fire wood (gC)
    REAL(r_std), DIMENSION(nelements)                    :: timber_biomass    !! Harvest with larger dimensions that will
                                                                              !! be used for all kinds of uses (gC)
    REAL(r_std), DIMENSION(nelements)                    :: residual          !! Residual @tex $(gC m^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,nmbcomp,nelements)   :: check_intern      !! Contains the components of the internal
                                                                              !! mass balance chech for this routine
                                                                              !! @tex $(gC m^{-2} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)           :: closure_intern    !! Check closure of internal mass balance
                                                                              !! @tex $(gC m^{-2} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)           :: pool_start        !! Start pool of this routine 
                                                                              !! @tex $(gC m^{-2} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)           :: pool_end          !! End pool of this routine 
                                                                              !! @tex $(gC m^{-2} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts)                         :: pixel_area        !! surface area of the pixel @tex ($m^{2}$) @endtex
!_ ================================================================================================================================    

    IF (printlev.GE.3) WRITE(numout,*) 'Entering dimensional product use'
 

 !! 1. initialization

    !! 1.1 Fluxes and pools
    prod_s(:,0,:) = zero
    prod_m(:,0,:) = zero
    prod_l(:,0,:) = zero
    prod_s_total(:,:) = zero
    prod_m_total(:,:) = zero
    prod_l_total(:,:) = zero   
    flux_prod_s(:,:) = zero
    flux_prod_m(:,:) = zero
    flux_prod_l(:,:) = zero
    flux_prod_total(:,:) = zero

    !! 1.2 Pixel area
    !  Surface area (m2) of the pixel
    pixel_area(:) = resolution(:,1) * resolution(:,2)

    !! 1.3 Initialize local printlev
    !printlev_loc=get_printlev('product_use')

    !! 1.4 Initialize check for mass balance closure 
    IF (err_act.GT.1) THEN

       ! The biomass harvest pool shouldn't be multiplied by veget_cov_max
       ! its units are already in gC pixel-1. The total amount for the 
       ! pixel was stored in harvest_pool. Account for the harvest and
       ! wood product pools. There are no longer PFTs
       check_intern(:,:,:,:) = zero
       pool_start(:,:,:) = zero
       DO iele = 1,nelements
          pool_start(:,1,iele) = &
               ( SUM(SUM(harvest_pool(:,:,:,iele),3),2) + &
               SUM(prod_l(:,:,iele),2) + SUM(prod_m(:,:,iele),2) + &
               SUM(prod_s(:,:,iele),2) )/pixel_area(:)
       ENDDO
       
    ENDIF ! err_act.GT.1

    !! 1.4 Logical check of coeff_lcchange
    !  Within a PFT the coeff_lcchange should add up to one. Else the 
    !  carbon balance will not be closed! Do not include PFT 1 in this 
    !  check because its coeff_lcchange were set to undef (-9999).
    DO ivm = 2,nvm

       IF ( (coeff_lcchange_s(ivm) + coeff_lcchange_m(ivm) + &
            coeff_lcchange_l(ivm) .LT. un - min_stomate) .OR. &
            (coeff_lcchange_s(ivm) + coeff_lcchange_m(ivm) + &
            coeff_lcchange_l(ivm) .GT. un + min_stomate) ) THEN

          WRITE(numout,*) 'Error: coeff_lcchange in sapiens_product_use.f90 ' //&
               'do not add up to 1 in PFT - cannot close the mass balance, ',ivm
          CALL ipslerr_p (3,'sapiens_product_use', &
               'dim_product_use','coeff_lcchange do not add up to 1 ','')
         
       ENDIF

    ENDDO


 !! 2. Partition harvest in a short, medium and long-lived product pool

    DO ipts = 1,npts

       !! 2.1 Calculate the initial mass of the product pool (gC)
       DO ivm = 2,nvm

          !+++NOTE+++
          ! For the moment all harvest is lumped together and we don't 
          ! make use of harvest_type and harvest_cut to refine the product
          ! use. If these variables are to be used be aware that for the
          ! moment these values can be overwritten within a simulation.
          ! Although the sequence of the calls in stomate_lpj is believed
          ! to protect against this, it is not enforced by the code. For
          ! example, if we first thin and then harvest, wood of both
          ! actions will be stored in the harvest pool but it will be 
          ! recored as just harvest (thinning will be overwritten by 
          ! harvest and no warning will be given)
          !++++++++++
          fuelwood_biomass(:) = zero
          timber_biomass(:) = zero
 
          ! Grasses and crops
          IF (fuelwood_diameter(ivm) .LT. min_stomate) THEN

             !Grasses and crops do not provide fuelwood. They do have
             ! a very short turnover time. The values stored in 
             ! harvest_pool are absolute numbers gC (for that pixel), hence, 
             ! the spatial extent of the LCC change, harvest or thinning has 
             ! already been accounted for.
             exported_biomass(:) = SUM(harvest_pool(ipts,ivm,:,:),1)
             prod_s(ipts,0,:) = prod_s(ipts,0,:) + &
               (coeff_lcchange_s(ivm) * exported_biomass(:))
             prod_m(ipts,0,:) = prod_m(ipts,0,:) + &
               (coeff_lcchange_m(ivm) * exported_biomass(:))
             prod_l(ipts,0,:) = prod_l(ipts,0,:) + &
               (coeff_lcchange_l(ivm) * exported_biomass(:))

          ! Forests
          ELSE

             ! This is a woody PFT, check the diameter of the harvested wood
             timber_biomass(:) = SUM(harvest_pool(ipts,ivm,:,:),1)
             DO idia = 1,ndia_harvest

                IF (harvest_pool_bound(idia) .LT. fuelwood_diameter(ivm)) THEN

                   ! If the diameter of the harvest pool is below the treshold
                   ! the wood is used as fuelwood and will end up in the short
                   ! lived pool
                   fuelwood_biomass(:) = fuelwood_biomass(:) + & 
                        harvest_pool(ipts,ivm,idia,:)

                ENDIF

             ENDDO ! harvest diameter

             ! Calculate the long lived pools as the residual of all
             ! wood mines the fuel wood.
             timber_biomass(:) = timber_biomass(:) - fuelwood_biomass(:)

             ! Attribute the harvested biomass to pools with a different 
             ! turnover time. The turnover times are PFT depend which 
             ! accounts for the different uses different PFT's are put to i.e. 
             ! wood from forests vs grass or crops. The values stored in 
             ! harvest_pool are absolute numbers gC (for that pixel), hence, 
             ! the spatial extent of the LCC change, harvest or thinning has 
             ! already been accounted for.
             prod_s(ipts,0,:) = prod_s(ipts,0,:) + fuelwood_biomass(:)
             IF  (coeff_lcchange_m(ivm) + coeff_lcchange_l(ivm) .GT. &
                  min_stomate) THEN

                ! If the condition is satisfied, calculate the fractions
                ! if it is not satisfied there is no medium and long lived
                ! pool for the products. Nothing should then be done. Avoid
                ! divide by zero
                prod_m(ipts,0,:) = prod_m(ipts,0,:) + &
                     (coeff_lcchange_m(ivm)/ &
                     (coeff_lcchange_m(ivm)+coeff_lcchange_l(ivm)) * &
                     timber_biomass(:))
                prod_l(ipts,0,:) = prod_l(ipts,0,:) + &
                     (coeff_lcchange_l(ivm)/ &
                     (coeff_lcchange_m(ivm)+coeff_lcchange_l(ivm)) * &
                     timber_biomass(:))

             ENDIF ! Check coeff_lcchange

          ENDIF ! grasses/crops vs forests

       ENDDO ! loop over PFTs 

       !! 2.2 Check whether all the harvest was used as is to be expected
       !  The precision we are aiming for is 10-8 per m-2. The harvest pool
       !  is expressed for the whole pixel
       residual(:) = SUM(SUM(harvest_pool(ipts,:,:,:),2),1) - &
            prod_s(ipts,0,:) - prod_m(ipts,0,:) - &
            prod_l(ipts,0,:)
      
       DO iele = 1,nelements

          IF (residual(iele)/pixel_area(ipts) .GT. min_stomate .OR. &
               residual(iele)/pixel_area(ipts) .LT. - min_stomate) THEN
             
             WRITE (numout,*) 'Error: some harvest in '//&
                  'sapiens_product_use has not been attributed'
             WRITE (numout,*) 'Residual in pixel, iele, ', &
                  iele, ipts, residual
             CALL ipslerr_p (3,'sapiens_product_use', &
                  'basic_product_use','some harvest has not been attributed',&
                  '')
        
          ELSE

             ! We could still have residual within the precision at the 
             ! m2 level but outside the precision at the pixel level. 
             ! Move the residual to the short lived pool
             prod_s(ipts,0,iele) = prod_s(ipts,0,iele) + residual(iele)

             ! All the carbon in the harvest pool was allocated to the product 
             ! pools the harvest pool can be set to zero so we can us it with 
             ! confidence in the mass balance calculation
             harvest_pool(ipts,:,:,iele) = zero
             harvest_type(ipts,:) = zero
             harvest_cut(ipts,:) = zero
             harvest_area(ipts,:) = zero

          ENDIF 

       ENDDO

              !! 2.3 Update the long-turnover pool content
       !  Update the long-turnover pool content following flux emission
       !  of a linear decay (1/longivety) of the initial carbon input.
       !  This must be implemented with a counter going from ::nlong to 
       !  zero because this way it takes ::nlong years before the intial 
       !  pool has been entirely consumed. If a more intuitive approach 
       !  would have been used in which the counter goes from zero to
       !  ::nlong, the pool would be depleted in the first year.
       !  Update the long-turnover pool content following flux emission.
       DO iage = nlong,2,-1

          ! flux and prod are large numbers. At the last time step 10-5
          ! gC remains which is outside the precision. Therefore, this 
          ! carbon is moved to the flux 
          IF (iage .EQ. nlong) THEN
              
              flux_prod_l(ipts,:) = flux_prod_l(ipts,:) + prod_l(ipts,iage-1,:)
              flux_l(ipts,iage,:) = prod_l(ipts,iage-1,:)
              prod_l(ipts,iage,:) = zero

          ELSE
          
             ! ::flux_l has INTENT(inout), it thus has the value from the 
             ! previuous year at the start of these calculations. Copy
             ! the flux from the previous year to this year and update
             ! the product pool
             flux_prod_l(ipts,:) = flux_prod_l(ipts,:) + flux_l(ipts,iage,:)
             prod_l(ipts,iage,:) = prod_l(ipts,iage-1,:) - flux_l(ipts,iage-1,:)
             flux_l(ipts,iage,:) = flux_l(ipts,iage-1,:)
 
          ENDIF
          
       ENDDO

       ! Treat the first year separately because it is not included in 
       ! the loop above
       flux_prod_l(ipts,:) = flux_prod_l(ipts,:) + flux_l(ipts,1,:)

       ! Each year 1/::nlong of the initial pool is decomposed. This fixed
       ! amount of decomposition is moved to a higher age class each year. 
       ! The units of ::flux_m are gC (absolute number as is the product 
       ! pool). At this point in the calculation, the flux is NOT a flux 
       ! yet but is also a pool 
       flux_l(ipts,1,:) = (un/nlong) * prod_l(ipts,0,:)
       prod_l(ipts,1,:) = prod_l(ipts,0,:) - flux_l(ipts,1,:)
       prod_l(ipts,0,:) = zero

       !! 2.4 Update the medium-turnover pool content 
       !  Update the medium-turnover pool content following flux emission.
       !  For comments see the section above. The concept is identical. 
       DO iage = nmedium,2,-1

          IF (iage .EQ. nmedium) THEN
              
              flux_prod_m(ipts,:) = flux_prod_m(ipts,:) + prod_m(ipts,iage-1,:)
              flux_m(ipts,iage,:) = prod_m(ipts,iage-1,:)
              prod_m(ipts,iage,:) = zero

          ELSE

             flux_prod_m(ipts,:) = flux_prod_m(ipts,:) + flux_m(ipts,iage,:)
             prod_m(ipts,iage,:) = prod_m(ipts,iage-1,:) - flux_m(ipts,iage-1,:)
             flux_m(ipts,iage,:) = flux_m(ipts,iage-1,:)
         
          ENDIF

       ENDDO
     
       flux_prod_m(ipts,:) = flux_prod_m(ipts,:) + flux_m(ipts,1,:)
       flux_m(ipts,1,:) = (un/nmedium) * prod_m(ipts,0,:)
       prod_m(ipts,1,:) = prod_m(ipts,0,:) - flux_m(ipts,1,:)
       prod_m(ipts,0,:) = zero

       !! 2.5 Update the short-turnover pool
       !  The length of the short-turnover pool will most likely be
       !  less than length of the regular loop. The regular loop is
       !  now conditional. 
       !  NOTE: this code has only been tested for nshort = 1 and 
       !  nshort = 2
       IF (nshort .GE. 4) THEN

          DO iage = nshort,2,-1

             IF (iage .EQ. nshort) THEN
              
                flux_prod_s(ipts,:) = flux_prod_s(ipts,:) + &
                     prod_s(ipts,iage-1,:)
                flux_s(ipts,iage,:) = prod_s(ipts,iage-1,:)
                prod_s(ipts,iage,:) = zero

             ELSE
  
                flux_prod_s(ipts,:) = flux_prod_s(ipts,:) + &
                     flux_s(ipts,iage,:)
                prod_s(ipts,iage,:) = prod_s(ipts,iage-1,:) - &
                     flux_s(ipts,iage-1,:)
                flux_s(ipts,iage,:) = flux_s(ipts,iage-1,:)

             ENDIF

          ENDDO

          flux_prod_s(ipts,:) = flux_prod_s(ipts,:) + flux_s(ipts,1,:) 
          flux_s(ipts,1,:) = (un/nshort) * prod_s(ipts,0,:)
          prod_s(ipts,1,:) = prod_s(ipts,0,:) - flux_s(ipts,1,:)
          prod_s(ipts,0,:) = zero

       ELSEIF (nshort .EQ. 3) THEN

          ! Decompose the product pool from the last year
          flux_prod_s(ipts,:) = flux_prod_s(ipts,:) + prod_s(ipts,2,:)
          flux_s(ipts,3,:) = prod_s(ipts,2,:)
          prod_s(ipts,3,:) = zero
          flux_prod_s(ipts,:) = flux_prod_s(ipts,:) + flux_s(ipts,2,:)
          prod_s(ipts,2,:) = prod_s(ipts,1,:) - flux_s(ipts,1,:)
          flux_s(ipts,2,:) = flux_s(ipts,1,:)

          ! Calculate the linear product decomposition for this
          ! year's harvest
          flux_prod_s(ipts,:) = flux_prod_s(ipts,:) + flux_s(ipts,1,:) 
          flux_s(ipts,1,:) = (un/nshort) * prod_s(ipts,0,:)
          prod_s(ipts,1,:) = prod_s(ipts,0,:) - flux_s(ipts,1,:)
          prod_s(ipts,0,:) = zero

       ELSEIF (nshort .EQ. 2) THEN

          ! Decompose the product pool from last year
          flux_prod_s(ipts,:) = flux_prod_s(ipts,:) + prod_s(ipts,1,:)
          flux_s(ipts,2,:) = prod_s(ipts,1,:)
          prod_s(ipts,2,:) = zero

          ! Calculate the linear product decomposition for this
          ! year's harvest
          flux_prod_s(ipts,:) = flux_prod_s(ipts,:) + flux_s(ipts,1,:) 
          flux_s(ipts,1,:) = (un/nshort) * prod_s(ipts,0,:)
          prod_s(ipts,1,:) = prod_s(ipts,0,:) - flux_s(ipts,1,:)
          prod_s(ipts,0,:) = zero

       ELSEIF (nshort .EQ. 1) THEN

          ! Everything decomposes in a single year. The short product
          ! pool remains empty.
          flux_prod_s(ipts,:) = flux_prod_s(ipts,:) + prod_s(ipts,0,:) 
          flux_s(ipts,1,:) = prod_s(ipts,0,:)
          prod_s(ipts,1,:) = zero
          prod_s(ipts,0,:) = zero

       ELSE

          WRITE(numout,*) 'Error: flaw in the logic of IF-construct '//&
               '(basic_product_use in sapiens_product_use.f90)'
          CALL ipslerr_p(3,'ERROR: product_use',&
               'Logic flaw in if-construct','','')
          
       ENDIF ! number of ages in short-turnover product pool

    ENDDO ! #pixels - spatial domain  


 !! 3. Check numerical consistency of this routine

    IF (err_act.GT.1) THEN

       ! 3.1 Mass balance closure 
       ! 3.1.1 Calculate final biomass
       ! The biomass harvest pool shouldn't be multiplied by veget_cov_max
       ! its in gC pixel-1 so divide by pixel_area to get a value in
       ! gC m-2. Account for the harvest and wood product pools. The harvest 
       ! pool should be empty, it was added for completeness.
       pool_end = zero
       DO iele=1,nelements
          pool_end(:,1,iele) = &
               ( SUM(SUM(harvest_pool(:,:,:,iele),3),2) + &
               SUM(prod_l(:,:,iele),2) + SUM(prod_m(:,:,iele),2) + &
               SUM(prod_s(:,:,iele),2) ) / pixel_area(:)
       ENDDO

       !! 3.1.2 Calculate mass balance
       ! Common processes
       DO iele=1,nelements
          check_intern(:,1,iland2atm,iele) = &
               -un * ( SUM(flux_s(:,:,iele),2) + &
               SUM(flux_m(:,:,iele),2) + SUM(flux_l(:,:,iele),2) ) / &
               pixel_area(:)
          check_intern(:,1,ipoolchange,iele) = &
               -un * (pool_end(:,1,iele) - &
               pool_start(:,1,iele))
       ENDDO

       closure_intern = zero
       DO imbc = 1,nmbcomp
          DO iele=1,nelements
             closure_intern(:,1,iele) = closure_intern(:,1,iele) + &
                  check_intern(:,1,imbc,iele)
          ENDDO
       ENDDO

       ! 3.1.3 Check mass balance closure
       CALL check_mass_balance("dimensional_product_use", closure_intern, &
            npts, pool_end, pool_start, veget_cov_max, 'products')
      
    ENDIF ! err_act.GT.1

 !! 4. Convert the pools into fluxes and aggregate

    !  Convert pools into fluxes
    !  The pools are given in the absolute amount of carbon for the 
    !  whole pixel and the harvest was accumulated over the year. 
    !  The unit is thus gc pixel-1 year-1. These fluxes should be 
    !  expressed as gC m-2 dt-1. Therefore, divide by the resolution 
    !  of the pixel and the number of time steps within a year
    DO iele = 1,nelements
       flux_prod_s(:,iele) = flux_prod_s(:,iele) / one_year * &
            dt_days / pixel_area(:) 
       flux_prod_m(:,iele) = flux_prod_m(:,iele) / one_year * &
            dt_days / pixel_area(:) 
       flux_prod_l(:,iele) = flux_prod_l(:,iele) / one_year * &
            dt_days / pixel_area(:) 

       ! Calculate aggregate values
       ! flux_prod_total is now in gC m-2 dt-1. The prod_x_total 
       ! are in gC m-2 
       flux_prod_total(:,iele) = flux_prod_s(:,iele) + &
            flux_prod_m(:,iele) + flux_prod_l(:,iele)
       prod_s_total(:,iele) = SUM(prod_s(:,:,iele),2) / pixel_area(:)
       prod_m_total(:,iele) = SUM(prod_m(:,:,iele),2) / pixel_area(:) 
       prod_l_total(:,iele) = SUM(prod_l(:,:,iele),2) / pixel_area(:) 
       
    ENDDO

    IF (printlev.GE.4) WRITE(numout,*) 'Leaving dimensional product use'


  END SUBROUTINE dim_product_use

!! ================================================================================================================================
!! SUBROUTINE   : product_init
!!
!>\BRIEF        Initialize the product use variables when product use is not
!!              calculated 
!!
!! DESCRIPTION  : Initialize the product use variables when product use is not
!! calculated
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S) : prod_s, prod_m, prod_l, flux_s, flux_m, flux_l,
!!   flux_prod_s flux_prod_m and flux_prod1_l 
!!
!! REFERENCES   : None
!!
!! FLOWCHART    : None
!!
!_ ================================================================================================================================

 SUBROUTINE product_init(flux_prod_s, flux_prod_m, flux_prod_l, &
               prod_s_total, prod_m_total, prod_l_total, flux_s, &
               flux_m, flux_l, flux_prod_total)

   IMPLICIT NONE


 !! 0. Variable and parameter declaration 
    
    !! 0.1 Input variables

    !! 0.2 Output variables

    REAL(r_std), DIMENSION(:,:), INTENT(out)              :: flux_prod_s      !! C-released during first years (short term) 
                                                                              !! following land cover change 
                                                                              !! @tex ($gC m^{-1} dt^{-1}$) @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(out)              :: flux_prod_m      !! Total annual release from decomposition of 
                                                                              !! the medium-lived product pool 
                                                                              !! @tex ($gC m^{-1} dt^{-1}$) @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(out)              :: flux_prod_l      !! Total annual release from decomposition of 
                                                                              !! the long-lived product pool 
                                                                              !! @tex ($gC m^{-1} dt^{-1}$) @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(out)             :: prod_s_total      !! Total products remaining in the short-lived  
                                                                              !! pool after the annual decomposition 
                                                                              !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(out)             :: prod_m_total      !! Total products remaining in the medium-lived
                                                                              !! pool after the annual decomposition 
                                                                              !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(out)             :: prod_l_total      !! Total products remaining in the long-lived
                                                                              !! pool after the annual release 
                                                                              !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(out)              :: flux_prod_total  !! Total flux from decomposition of the short,
                                                                              !! medium and long lived product pools 
                                                                              !! @tex $(gC m^{-1} dt^{-1})$ @endtex

    !! 0.3 Modified variables

    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)         :: flux_s            !! Annual release from the short-lived product 
                                                                              !! pool @tex $(gC) pixel^{-1} year^{-1}$ @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)         :: flux_m            !! Annual release from the medium-lived product  
                                                                              !! pool @tex $(gC) pixel^{-1} year^{-1}$ @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)         :: flux_l            !! Annual release from the long-lived product
                                                                              !! pool @tex $(gC) pixel^{-1} year^{-1}$ @endtex

    !! 0.4 Local variables

!_ ================================================================================================================================    

    IF (printlev.GE.3) WRITE(numout,*) 'Entering basic product init'
 
 !! 1. initialization

    flux_prod_s(:,:) = zero
    flux_prod_m(:,:) = zero
    flux_prod_l(:,:) = zero
    prod_s_total(:,:) = zero
    prod_m_total(:,:) = zero
    prod_l_total(:,:) = zero
    flux_s(:,:,:) = zero
    flux_m(:,:,:) = zero
    flux_l(:,:,:) = zero
    flux_prod_total(:,:) = zero

    IF (printlev.GE.4) WRITE(numout,*) 'Leaving basic product init'

  END SUBROUTINE product_init

END MODULE sapiens_product_use
